@@ -10,34 +10,34 @@
 #define SIGFRAME_MAX_OFFSET 8
 
 struct rt_sigcontext {
-	unsigned long			r8;
-	unsigned long			r9;
-	unsigned long			r10;
-	unsigned long			r11;
-	unsigned long			r12;
-	unsigned long			r13;
-	unsigned long			r14;
-	unsigned long			r15;
-	unsigned long			rdi;
-	unsigned long			rsi;
-	unsigned long			rbp;
-	unsigned long			rbx;
-	unsigned long			rdx;
-	unsigned long			rax;
-	unsigned long			rcx;
-	unsigned long			rsp;
-	unsigned long			rip;
-	unsigned long			eflags;
-	unsigned short			cs;
-	unsigned short			gs;
-	unsigned short			fs;
-	unsigned short			ss;
-	unsigned long			err;
-	unsigned long			trapno;
-	unsigned long			oldmask;
-	unsigned long			cr2;
-	void				*fpstate;
-	unsigned long			reserved1[8];
+	uint64_t			r8;
+	uint64_t			r9;
+	uint64_t			r10;
+	uint64_t			r11;
+	uint64_t			r12;
+	uint64_t			r13;
+	uint64_t			r14;
+	uint64_t			r15;
+	uint64_t			rdi;
+	uint64_t			rsi;
+	uint64_t			rbp;
+	uint64_t			rbx;
+	uint64_t			rdx;
+	uint64_t			rax;
+	uint64_t			rcx;
+	uint64_t			rsp;
+	uint64_t			rip;
+	uint64_t			eflags;
+	uint16_t			cs;
+	uint16_t			gs;
+	uint16_t			fs;
+	uint16_t			ss;
+	uint64_t			err;
+	uint64_t			trapno;
+	uint64_t			oldmask;
+	uint64_t			cr2;
+	uint64_t			fpstate;
+	uint64_t			reserved1[8];
 };
 
 struct rt_sigcontext_32 {
