@@ -97,7 +97,8 @@ int __attribute__((weak)) arch_set_thread_regs(struct pstree_item *item,
 	return 0;
 }
 
-static char loc_buf[PAGE_SIZE];
+#define PERSONALITY_LENGTH	9
+static char loc_buf[PERSONALITY_LENGTH];
 
 void free_mappings(struct vm_area_list *vma_area_list)
 {
@@ -715,6 +716,8 @@ int dump_thread_core(int pid, CoreEntry *core, const struct parasite_dump_thread
 			tc->pdeath_sig = ti->pdeath_sig;
 		}
 	}
+	if (!ret)
+		ret = seccomp_dump_thread(pid, tc);
 
 	return ret;
 }
@@ -728,7 +731,6 @@ static int dump_task_core_all(struct parasite_ctl *ctl,
 	CoreEntry *core = item->core[0];
 	pid_t pid = item->pid->real;
 	int ret = -1;
-	struct proc_status_creds *creds;
 	struct parasite_dump_cgroup_args cgroup_args, *info = NULL;
 
 	BUILD_BUG_ON(sizeof(cgroup_args) < PARASITE_ARG_SIZE_MIN);
@@ -741,18 +743,6 @@ static int dump_task_core_all(struct parasite_ctl *ctl,
 	if (ret < 0)
 		goto err;
 
-	creds = dmpi(item)->pi_creds;
-	if (creds->s.seccomp_mode != SECCOMP_MODE_DISABLED) {
-		pr_info("got seccomp mode %d for %d\n", creds->s.seccomp_mode, vpid(item));
-		core->tc->has_seccomp_mode = true;
-		core->tc->seccomp_mode = creds->s.seccomp_mode;
-
-		if (creds->s.seccomp_mode == SECCOMP_MODE_FILTER) {
-			core->tc->has_seccomp_filter = true;
-			core->tc->seccomp_filter = creds->last_filter;
-		}
-	}
-
 	strlcpy((char *)core->tc->comm, stat->comm, TASK_COMM_LEN);
 	core->tc->flags = stat->flags;
 	core->tc->task_state = item->pid->state;
@@ -1137,7 +1127,7 @@ err:
 	return ret;
 }
 
-static int pre_dump_one_task(struct pstree_item *item)
+static int pre_dump_one_task(struct pstree_item *item, InventoryEntry *parent_ie)
 {
 	pid_t pid = item->pid->real;
 	struct vm_area_list vmas;
@@ -1196,6 +1186,8 @@ static int pre_dump_one_task(struct pstree_item *item)
 
 	mdc.pre_dump = true;
 	mdc.lazy = false;
+	mdc.stat = NULL;
+	mdc.parent_ie = parent_ie;
 
 	ret = parasite_dump_pages_seized(item, &vmas, &mdc, parasite_ctl);
 	if (ret)
@@ -1214,7 +1206,7 @@ err_cure:
 	goto err_free;
 }
 
-static int dump_one_task(struct pstree_item *item)
+static int dump_one_task(struct pstree_item *item, InventoryEntry *parent_ie)
 {
 	pid_t pid = item->pid->real;
 	struct vm_area_list vmas;
@@ -1356,6 +1348,8 @@ static int dump_one_task(struct pstree_item *item)
 
 	mdc.pre_dump = false;
 	mdc.lazy = opts.lazy_pages;
+	mdc.stat = &pps_buf;
+	mdc.parent_ie = parent_ie;
 
 	ret = parasite_dump_pages_seized(item, &vmas, &mdc, parasite_ctl);
 	if (ret)
@@ -1471,6 +1465,7 @@ static int setup_alarm_handler()
 
 static int cr_pre_dump_finish(int ret)
 {
+	InventoryEntry he = INVENTORY_ENTRY__INIT;
 	struct pstree_item *item;
 
 	/*
@@ -1479,6 +1474,8 @@ static int cr_pre_dump_finish(int ret)
 	 */
 	if (arch_set_thread_regs(root_item, false) < 0)
 		goto err;
+
+	prepare_inventory_pre_dump(&he);
 	pstree_switch_state(root_item, TASK_ALIVE);
 
 	timing_stop(TIME_FROZEN);
@@ -1513,6 +1510,7 @@ static int cr_pre_dump_finish(int ret)
 	}
 
 	free_pstree(root_item);
+	seccomp_free_entries();
 
 	if (irmap_predump_run()) {
 		ret = -1;
@@ -1526,6 +1524,9 @@ err:
 	if (bfd_flush_images())
 		ret = -1;
 
+	if (write_img_inventory(&he))
+		ret = -1;
+
 	if (ret)
 		pr_err("Pre-dumping FAILED.\n");
 	else {
@@ -1537,6 +1538,7 @@ err:
 
 int cr_pre_dump_tasks(pid_t pid)
 {
+	InventoryEntry *parent_ie = NULL;
 	struct pstree_item *item;
 	int ret = -1;
 
@@ -1588,10 +1590,18 @@ int cr_pre_dump_tasks(pid_t pid)
 	if (collect_namespaces(false) < 0)
 		goto err;
 
+	/* Errors handled later in detect_pid_reuse */
+	parent_ie = get_parent_inventory();
+
 	for_each_pstree_item(item)
-		if (pre_dump_one_task(item))
+		if (pre_dump_one_task(item, parent_ie))
 			goto err;
 
+	if (parent_ie) {
+		inventory_entry__free_unpacked(parent_ie, NULL);
+		parent_ie = NULL;
+	}
+
 	ret = cr_dump_shmem();
 	if (ret)
 		goto err;
@@ -1601,6 +1611,9 @@ int cr_pre_dump_tasks(pid_t pid)
 
 	ret = 0;
 err:
+	if (parent_ie)
+		inventory_entry__free_unpacked(parent_ie, NULL);
+
 	return cr_pre_dump_finish(ret);
 }
 
@@ -1695,6 +1708,7 @@ static int cr_dump_finish(int ret)
 			    TASK_ALIVE : opts.final_state);
 	timing_stop(TIME_FROZEN);
 	free_pstree(root_item);
+	seccomp_free_entries();
 	free_file_locks();
 	free_link_remaps();
 	free_aufs_branches();
@@ -1714,6 +1728,7 @@ static int cr_dump_finish(int ret)
 int cr_dump_tasks(pid_t pid)
 {
 	InventoryEntry he = INVENTORY_ENTRY__INIT;
+	InventoryEntry *parent_ie = NULL;
 	struct pstree_item *item;
 	int pre_dump_ret = 0;
 	int ret = -1;
@@ -1798,14 +1813,22 @@ int cr_dump_tasks(pid_t pid)
 	if (!glob_imgset)
 		goto err;
 
-	if (collect_seccomp_filters() < 0)
+	if (seccomp_collect_dump_filters() < 0)
 		goto err;
 
+	/* Errors handled later in detect_pid_reuse */
+	parent_ie = get_parent_inventory();
+
 	for_each_pstree_item(item) {
-		if (dump_one_task(item))
+		if (dump_one_task(item, parent_ie))
 			goto err;
 	}
 
+	if (parent_ie) {
+		inventory_entry__free_unpacked(parent_ie, NULL);
+		parent_ie = NULL;
+	}
+
 	/*
 	 * It may happen that a process has completed but its files in
 	 * /proc/PID/ are still open by another process. If the PID has been
@@ -1841,9 +1864,11 @@ int cr_dump_tasks(pid_t pid)
 	if (ret)
 		goto err;
 
-	if (root_ns_mask)
-		if (dump_namespaces(root_item, root_ns_mask) < 0)
+	if (root_ns_mask) {
+		ret = dump_namespaces(root_item, root_ns_mask);
+		if (ret)
 			goto err;
+	}
 
 	ret = dump_cgroups();
 	if (ret)
@@ -1861,5 +1886,8 @@ int cr_dump_tasks(pid_t pid)
 	if (ret)
 		goto err;
 err:
+	if (parent_ie)
+		inventory_entry__free_unpacked(parent_ie, NULL);
+
 	return cr_dump_finish(ret);
 }
