@@ -24,7 +24,7 @@
 
 extern int log_init(const char *output);
 extern void log_fini(void);
-extern int log_init_by_pid(void);
+extern int log_init_by_pid(pid_t pid);
 extern void log_closedir(void);
 extern int log_keep_err(void);
 extern char *log_first_err(void);
@@ -34,6 +34,7 @@ extern int log_get_fd(void);
 
 extern void log_set_loglevel(unsigned int loglevel);
 extern unsigned int log_get_loglevel(void);
+extern void log_get_logstart(struct timeval *);
 
 extern int write_pidfile(int pid);
 
