@@ -12,11 +12,6 @@
 #include "log.h"
 #include "common/bug.h"
 
-#ifdef _ARCH_PPC64
-#define PAGE_IMAGE_SIZE	65536
-#else
-#define PAGE_IMAGE_SIZE	4096
-#endif /* _ARCH_PPC64 */
 #define PAGE_RSS	1
 #define PAGE_ANON	2
 
@@ -89,6 +84,9 @@
 #define VMA_AREA_VVAR		(1 <<  12)
 #define VMA_AREA_AIORING	(1 <<  13)
 
+#define VMA_CLOSE		(1 <<  28)
+#define VMA_NO_PROT_WRITE	(1 <<  29)
+#define VMA_PREMMAPED		(1 <<  30)
 #define VMA_UNSUPP		(1 <<  31)
 
 #define CR_CAP_SIZE	2
