@@ -5,7 +5,7 @@
 #ifndef __CR_SIGFRAME_H__
 #define __CR_SIGFRAME_H__
 
-#include "asm/sigframe.h"
+#include <compel/asm/sigframe.h>
 #include "images/core.pb-c.h"
 
 extern int construct_sigframe(struct rt_sigframe *sigframe,
