@@ -290,6 +290,46 @@ static int xfer_pages(struct page_pipe *pp, struct page_xfer *xfer)
 	return ret;
 }
 
+static int detect_pid_reuse(struct pstree_item *item,
+			    struct proc_pid_stat* pps,
+			    InventoryEntry *parent_ie)
+{
+	unsigned long long dump_ticks;
+	struct proc_pid_stat pps_buf;
+	unsigned long long tps; /* ticks per second */
+	int ret;
+
+	tps = sysconf(_SC_CLK_TCK);
+	if (tps == -1) {
+		pr_perror("Failed to get clock ticks via sysconf");
+		return -1;
+	}
+
+	if (!pps) {
+		pps = &pps_buf;
+		ret = parse_pid_stat(item->pid->real, pps);
+		if (ret < 0)
+			return -1;
+	}
+
+	if (!parent_ie) {
+		pr_err("Pid-reuse detection failed: no parent inventory, " \
+		       "check warnings in get_parent_stats\n");
+		return -1;
+	}
+
+	dump_ticks = parent_ie->dump_uptime/(USEC_PER_SEC/tps);
+
+	if (pps->start_time >= dump_ticks) {
+		/* Print "*" if unsure */
+		pr_warn("Pid reuse%s detected for pid %d\n",
+			pps->start_time == dump_ticks ? "*" : "",
+			item->pid->real);
+		return 1;
+	}
+	return 0;
+}
+
 static int __parasite_dump_pages_seized(struct pstree_item *item,
 		struct parasite_dump_pages_args *args,
 		struct vm_area_list *vma_area_list,
@@ -300,9 +340,10 @@ static int __parasite_dump_pages_seized(struct pstree_item *item,
 	struct page_pipe *pp;
 	struct vma_area *vma_area;
 	struct page_xfer xfer = { .parent = NULL };
-	int ret = -1;
+	int ret, exit_code = -1;
 	unsigned cpp_flags = 0;
 	unsigned long pmc_size;
+	int possible_pid_reuse = 0;
 
 	pr_info("\n");
 	pr_info("Dumping pages (type: %d pid: %d)\n", CR_FD_PAGES, item->pid->real);
@@ -323,7 +364,6 @@ static int __parasite_dump_pages_seized(struct pstree_item *item,
 			 pmc_size * PAGE_SIZE))
 		return -1;
 
-	ret = -1;
 	if (!(mdc->pre_dump || mdc->lazy))
 		/*
 		 * Chunk mode pushes pages portion by portion. This mode
@@ -357,6 +397,14 @@ static int __parasite_dump_pages_seized(struct pstree_item *item,
 			xfer.parent = NULL + 1;
 	}
 
+	if (xfer.parent) {
+		possible_pid_reuse = detect_pid_reuse(item, mdc->stat,
+						      mdc->parent_ie);
+		if (possible_pid_reuse == -1)
+			goto out_xfer;
+	}
+
+
 	/*
 	 * Step 1 -- generate the pagemap
 	 */
@@ -383,7 +431,7 @@ static int __parasite_dump_pages_seized(struct pstree_item *item,
 		else {
 again:
 			ret = generate_iovs(vma_area, pp, map, &off,
-				has_parent);
+				has_parent && !possible_pid_reuse);
 			if (ret == -EAGAIN) {
 				BUG_ON(!(pp->flags & PP_CHUNK_MODE));
 
@@ -416,6 +464,9 @@ again:
 	 */
 
 	ret = task_reset_dirty_track(item->pid->real);
+	if (ret)
+		goto out_xfer;
+	exit_code = 0;
 out_xfer:
 	if (!mdc->pre_dump)
 		xfer.close(&xfer);
@@ -427,7 +478,7 @@ out_pp:
 out:
 	pmc_fini(&pmc);
 	pr_info("----------------------------------------\n");
-	return ret;
+	return exit_code;
 }
 
 int parasite_dump_pages_seized(struct pstree_item *item,
