@@ -47,10 +47,12 @@
 #include "namespaces.h"
 #include "criu-log.h"
 
+#include "clone-noasan.h"
 #include "cr_options.h"
 #include "servicefd.h"
 #include "cr-service.h"
 #include "files.h"
+#include "pstree.h"
 
 #include "cr-errno.h"
 
@@ -291,11 +293,10 @@ int move_fd_from(int *img_fd, int want_fd)
  */
 
 static pid_t open_proc_pid = PROC_NONE;
-static int open_proc_fd = -1;
 static pid_t open_proc_self_pid;
 static int open_proc_self_fd = -1;
 
-static inline void set_proc_self_fd(int fd)
+void set_proc_self_fd(int fd)
 {
 	if (open_proc_self_fd >= 0)
 		close(open_proc_self_fd);
@@ -304,13 +305,18 @@ static inline void set_proc_self_fd(int fd)
 	open_proc_self_pid = getpid();
 }
 
-static inline void set_proc_pid_fd(int pid, int fd)
+static inline int set_proc_pid_fd(int pid, int fd)
 {
-	if (open_proc_fd >= 0)
-		close(open_proc_fd);
+	int ret;
+
+	if (fd < 0)
+		return close_service_fd(PROC_PID_FD_OFF);
 
 	open_proc_pid = pid;
-	open_proc_fd = fd;
+	ret = install_service_fd(PROC_PID_FD_OFF, fd);
+	close(fd);
+
+	return ret;
 }
 
 static inline int get_proc_fd(int pid)
@@ -322,7 +328,7 @@ static inline int get_proc_fd(int pid)
 		}
 		return open_proc_self_fd;
 	} else if (pid == open_proc_pid)
-		return open_proc_fd;
+		return get_service_fd(PROC_PID_FD_OFF);
 	else
 		return -1;
 }
@@ -406,7 +412,7 @@ inline int open_pid_proc(pid_t pid)
 	if (pid == PROC_SELF)
 		set_proc_self_fd(fd);
 	else
-		set_proc_pid_fd(pid, fd);
+		fd = set_proc_pid_fd(pid, fd);
 
 	return fd;
 }
@@ -428,7 +434,11 @@ int do_open_proc(pid_t pid, int flags, const char *fmt, ...)
 	return openat(dirfd, path, flags);
 }
 
-static int service_fd_rlim_cur;
+/* Max potentially possible fd to be open by criu process */
+int service_fd_rlim_cur;
+/* Base of current process service fds set */
+static int service_fd_base;
+/* Id of current process in shared fdt */
 static int service_fd_id = 0;
 
 int init_service_fd(void)
@@ -446,31 +456,40 @@ int init_service_fd(void)
 	}
 
 	service_fd_rlim_cur = (int)rlimit.rlim_cur;
-	BUG_ON(service_fd_rlim_cur < SERVICE_FD_MAX);
+	service_fd_base = service_fd_rlim_cur;
+	BUG_ON(service_fd_base < SERVICE_FD_MAX);
 
 	return 0;
 }
 
 static int __get_service_fd(enum sfd_type type, int service_fd_id)
 {
-	return service_fd_rlim_cur - type - SERVICE_FD_MAX * service_fd_id;
+	return service_fd_base - type - SERVICE_FD_MAX * service_fd_id;
 }
 
-int service_fd_min_fd(void)
+int service_fd_min_fd(struct pstree_item *item)
 {
-	return service_fd_rlim_cur - (SERVICE_FD_MAX - 1) - SERVICE_FD_MAX * service_fd_id;
+	struct fdt *fdt = rsti(item)->fdt;
+	int id = 0;
+
+	if (fdt)
+		id = fdt->nr - 1;
+	return service_fd_rlim_cur - (SERVICE_FD_MAX - 1) - SERVICE_FD_MAX * id;
 }
 
 static DECLARE_BITMAP(sfd_map, SERVICE_FD_MAX);
+/*
+ * Variable for marking areas of code, where service fds modifications
+ * are prohibited. It's used to safe them from reusing their numbers
+ * by ordinary files. See install_service_fd() and close_service_fd().
+ */
+bool sfds_protected = false;
 
-int reserve_service_fd(enum sfd_type type)
+static void sfds_protection_bug(enum sfd_type type)
 {
-	int sfd = __get_service_fd(type, service_fd_id);
-
-	BUG_ON((int)type <= SERVICE_FD_MIN || (int)type >= SERVICE_FD_MAX);
-
-	set_bit(type, sfd_map);
-	return sfd;
+	pr_err("Service fd %u is being modified in protected context\n", type);
+	print_stack_trace(current ? vpid(current) : 0);
+	BUG();
 }
 
 int install_service_fd(enum sfd_type type, int fd)
@@ -478,6 +497,8 @@ int install_service_fd(enum sfd_type type, int fd)
 	int sfd = __get_service_fd(type, service_fd_id);
 
 	BUG_ON((int)type <= SERVICE_FD_MIN || (int)type >= SERVICE_FD_MAX);
+	if (sfds_protected && !test_bit(type, sfd_map))
+		sfds_protection_bug(type);
 
 	if (dup3(fd, sfd, O_CLOEXEC) != sfd) {
 		pr_perror("Dup %d -> %d failed", fd, sfd);
@@ -507,6 +528,9 @@ int close_service_fd(enum sfd_type type)
 {
 	int fd;
 
+	if (sfds_protected)
+		sfds_protection_bug(type);
+
 	fd = get_service_fd(type);
 	if (fd < 0)
 		return 0;
@@ -518,27 +542,92 @@ int close_service_fd(enum sfd_type type)
 	return 0;
 }
 
-int clone_service_fd(int id)
+static void move_service_fd(struct pstree_item *me, int type, int new_id, int new_base)
 {
-	int ret = -1, i;
+	int old = get_service_fd(type);
+	int new = new_base - type - SERVICE_FD_MAX * new_id;
+	int ret;
 
-	if (service_fd_id == id)
-		return 0;
-
-	for (i = SERVICE_FD_MIN + 1; i < SERVICE_FD_MAX; i++) {
-		int old = get_service_fd(i);
-		int new = __get_service_fd(i, id);
-
-		if (old < 0)
-			continue;
-		ret = dup2(old, new);
-		if (ret == -1) {
-			if (errno == EBADF)
-				continue;
+	if (old < 0)
+		return;
+	ret = dup2(old, new);
+	if (ret == -1) {
+		if (errno != EBADF)
 			pr_perror("Unable to clone %d->%d", old, new);
+	} else if (!(rsti(me)->clone_flags & CLONE_FILES))
+		close(old);
+}
+
+static int choose_service_fd_base(struct pstree_item *me)
+{
+	int nr, real_nr, fdt_nr = 1, id = rsti(me)->service_fd_id;
+
+	if (rsti(me)->fdt) {
+		/* The base is set by owner of fdt (id 0) */
+		if (id != 0)
+			return service_fd_base;
+		fdt_nr = rsti(me)->fdt->nr;
+	}
+	/* Now find process's max used fd number */
+	if (!list_empty(&rsti(me)->fds))
+		nr = list_entry(rsti(me)->fds.prev,
+				struct fdinfo_list_entry, ps_list)->fe->fd;
+	else
+		nr = -1;
+
+	nr = max(nr, inh_fd_max);
+	/*
+	 * Service fds go after max fd near right border of alignment:
+	 *
+	 * ...|max_fd|max_fd+1|...|sfd first|...|sfd last (aligned)|
+	 *
+	 * So, they take maximum numbers of area allocated by kernel.
+	 * See linux alloc_fdtable() for details.
+	 */
+	nr += (SERVICE_FD_MAX - SERVICE_FD_MIN) * fdt_nr;
+	nr += 16; /* Safety pad */
+	real_nr = nr;
+
+	nr /= (1024 / sizeof(void *));
+	if (nr)
+		nr = 1 << (32 - __builtin_clz(nr));
+	else
+		nr = 1;
+	nr *= (1024 / sizeof(void *));
+
+	if (nr > service_fd_rlim_cur) {
+		/* Right border is bigger, than rlim. OK, then just aligned value is enough */
+		nr = round_down(service_fd_rlim_cur, (1024 / sizeof(void *)));
+		if (nr < real_nr) {
+			pr_err("Can't chose service_fd_base: %d %d\n", nr, real_nr);
+			return -1;
 		}
 	}
 
+	return nr;
+}
+
+int clone_service_fd(struct pstree_item *me)
+{
+	int id, new_base, i, ret = -1;
+
+	new_base = choose_service_fd_base(me);
+	id = rsti(me)->service_fd_id;
+
+	if (new_base == -1)
+		return -1;
+	if (service_fd_base == new_base && service_fd_id == id)
+		return 0;
+
+	/* Dup sfds in memmove() style: they may overlap */
+	if (get_service_fd(LOG_FD_OFF) < new_base - LOG_FD_OFF - SERVICE_FD_MAX * id)
+		for (i = SERVICE_FD_MIN + 1; i < SERVICE_FD_MAX; i++)
+			move_service_fd(me, i, id, new_base);
+	else
+		for (i = SERVICE_FD_MAX - 1; i > SERVICE_FD_MIN; i--)
+			move_service_fd(me, i, id, new_base);
+
+	service_fd_base = new_base;
 	service_fd_id = id;
 	ret = 0;
 
@@ -547,8 +636,16 @@ int clone_service_fd(int id)
 
 bool is_any_service_fd(int fd)
 {
-	return fd > __get_service_fd(SERVICE_FD_MAX, service_fd_id) &&
-		fd < __get_service_fd(SERVICE_FD_MIN, service_fd_id);
+	int sfd_min_fd = __get_service_fd(SERVICE_FD_MAX, service_fd_id);
+	int sfd_max_fd = __get_service_fd(SERVICE_FD_MIN, service_fd_id);
+
+	if (fd > sfd_min_fd && fd < sfd_max_fd) {
+		int type = SERVICE_FD_MAX - (fd - sfd_min_fd);
+		if (type > SERVICE_FD_MIN && type < SERVICE_FD_MAX)
+			return !!test_bit(type, sfd_map);
+	}
+
+	return false;
 }
 
 bool is_service_fd(int fd, enum sfd_type type)
@@ -1057,13 +1154,15 @@ const char *ns_to_string(unsigned int ns)
 void tcp_cork(int sk, bool on)
 {
 	int val = on ? 1 : 0;
-	setsockopt(sk, SOL_TCP, TCP_CORK, &val, sizeof(val));
+	if (setsockopt(sk, SOL_TCP, TCP_CORK, &val, sizeof(val)))
+		pr_perror("Unable to restore TCP_CORK (%d)", val);
 }
 
 void tcp_nodelay(int sk, bool on)
 {
 	int val = on ? 1 : 0;
-	setsockopt(sk, SOL_TCP, TCP_NODELAY, &val, sizeof(val));
+	if (setsockopt(sk, SOL_TCP, TCP_NODELAY, &val, sizeof(val)))
+		pr_perror("Unable to restore TCP_NODELAY (%d)", val);
 }
 
 static inline void pr_xsym(unsigned char *data, size_t len, int pos)
@@ -1138,39 +1237,50 @@ void print_data(unsigned long addr, unsigned char *data, size_t size)
 	}
 }
 
-static int get_sockaddr_in(struct sockaddr_in *addr, char *host)
+static int get_sockaddr_in(struct sockaddr_storage *addr, char *host)
 {
 	memset(addr, 0, sizeof(*addr));
-	addr->sin_family = AF_INET;
 
-	if (!host)
-		addr->sin_addr.s_addr = INADDR_ANY;
-	else if (!inet_aton(host, &addr->sin_addr)) {
+	if (!host) {
+ 		((struct sockaddr_in *)addr)->sin_addr.s_addr = INADDR_ANY;
+		addr->ss_family = AF_INET;
+	} else if (inet_pton(AF_INET, host, &((struct sockaddr_in *)addr)->sin_addr)) {
+		addr->ss_family = AF_INET;
+	} else if (inet_pton(AF_INET6, host, &((struct sockaddr_in6 *)addr)->sin6_addr)) {
+		addr->ss_family = AF_INET6;
+	} else {
 		pr_perror("Bad server address");
 		return -1;
 	}
 
-	addr->sin_port = opts.port;
+	if (addr->ss_family == AF_INET6) {
+		((struct sockaddr_in6 *)addr)->sin6_port = htons(opts.port);
+	} else if (addr->ss_family == AF_INET) {
+		((struct sockaddr_in *)addr)->sin_port = htons(opts.port);
+	}
+
 	return 0;
 }
 
 int setup_tcp_server(char *type)
 {
 	int sk = -1;
-	struct sockaddr_in saddr;
+	struct sockaddr_storage saddr;
 	socklen_t slen = sizeof(saddr);
 
-	pr_info("Starting %s server on port %u\n", type, (int)ntohs(opts.port));
+	if (get_sockaddr_in(&saddr, opts.addr)) {
+		return -1;
+	}
+
+	pr_info("Starting %s server on port %u\n", type, opts.port);
+
+	sk = socket(saddr.ss_family, SOCK_STREAM, IPPROTO_TCP);
 
-	sk = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 	if (sk < 0) {
 		pr_perror("Can't init %s server", type);
 		return -1;
 	}
 
-	if (get_sockaddr_in(&saddr, opts.addr))
-		goto out;
-
 	if (bind(sk, (struct sockaddr *)&saddr, slen)) {
 		pr_perror("Can't bind %s server", type);
 		goto out;
@@ -1188,7 +1298,12 @@ int setup_tcp_server(char *type)
 			goto out;
 		}
 
-		opts.port = ntohs(saddr.sin_port);
+		if (saddr.ss_family == AF_INET6) {
+			opts.port = ntohs(((struct sockaddr_in *)&saddr)->sin_port);
+		} else if (saddr.ss_family == AF_INET) {
+			opts.port = ntohs(((struct sockaddr_in6 *)&saddr)->sin6_port);
+		}
+
 		pr_info("Using %u port\n", opts.port);
 	}
 
@@ -1247,15 +1362,15 @@ out:
 
 int setup_tcp_client(char *addr)
 {
-	struct sockaddr_in saddr;
+	struct sockaddr_storage saddr;
 	int sk;
 
-	pr_info("Connecting to server %s:%u\n", addr, (int)ntohs(opts.port));
+	pr_info("Connecting to server %s:%u\n", addr, opts.port);
 
 	if (get_sockaddr_in(&saddr, addr))
 		return -1;
 
-	sk = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+	sk = socket(saddr.ss_family, SOCK_STREAM, IPPROTO_TCP);
 	if (sk < 0) {
 		pr_perror("Can't create socket");
 		return -1;
@@ -1377,3 +1492,50 @@ free_events:
 	xfree(*events);
 	return -1;
 }
+
+int call_in_child_process(int (*fn)(void *), void *arg)
+{
+	int status, ret = -1;
+	pid_t pid;
+	/*
+	 * Parent freezes till child exit, so child may use the same stack.
+	 * No SIGCHLD flag, so it's not need to block signal.
+	 */
+	pid = clone_noasan(fn, CLONE_VFORK | CLONE_VM | CLONE_FILES |
+			   CLONE_IO | CLONE_SIGHAND | CLONE_SYSVSEM, arg);
+	if (pid == -1) {
+		pr_perror("Can't clone");
+		return -1;
+	}
+	errno = 0;
+	if (waitpid(pid, &status, __WALL) != pid || !WIFEXITED(status) || WEXITSTATUS(status)) {
+		pr_err("Can't wait or bad status: errno=%d, status=%d\n", errno, status);
+		goto out;
+	}
+	ret = 0;
+	/*
+	 * Child opened PROC_SELF for pid. If we create one more child
+	 * with the same pid later, it will try to reuse this /proc/self.
+	 */
+out:
+	close_pid_proc();
+	return ret;
+}
+
+#ifdef __GLIBC__
+#include <execinfo.h>
+void print_stack_trace(pid_t pid)
+{
+	void *array[10];
+	char **strings;
+	size_t size, i;
+
+	size = backtrace(array, 10);
+	strings = backtrace_symbols(array, size);
+
+	for (i = 0; i < size; i++)
+		pr_err("stack %d#%zu: %s\n", pid, i, strings[i]);
+
+	free(strings);
+}
+#endif
