@@ -27,7 +27,7 @@ struct criu_opts {
 	union {
 		char		*service_address;
 		int		service_fd;
-		char		*service_binary;
+		const char	*service_binary;
 	};
 	int			swrk_pid;
 };
@@ -68,7 +68,7 @@ void criu_set_service_fd(int fd)
 	criu_local_set_service_fd(global_opts, fd);
 }
 
-void criu_local_set_service_binary(criu_opts *opts, char *path)
+void criu_local_set_service_binary(criu_opts *opts, const char *path)
 {
 	if (path)
 		opts->service_binary = path;
@@ -76,7 +76,7 @@ void criu_local_set_service_binary(criu_opts *opts, char *path)
 		opts->service_binary = CR_DEFAULT_SERVICE_BIN;
 }
 
-void criu_set_service_binary(char *path)
+void criu_set_service_binary(const char *path)
 {
 	criu_local_set_service_binary(global_opts, path);
 }
@@ -321,6 +321,17 @@ void criu_set_tcp_skip_in_flight(bool tcp_skip_in_flight)
 	criu_local_set_tcp_skip_in_flight(global_opts, tcp_skip_in_flight);
 }
 
+void criu_local_set_weak_sysctls(criu_opts *opts, bool val)
+{
+	opts->rpc->has_weak_sysctls = true;
+	opts->rpc->weak_sysctls	= val;
+}
+
+void criu_set_weak_sysctls(bool val)
+{
+	criu_local_set_weak_sysctls(global_opts, val);
+}
+
 void criu_local_set_evasive_devices(criu_opts *opts, bool evasive_devices)
 {
 	opts->rpc->has_evasive_devices	= true;
@@ -760,17 +771,21 @@ int criu_local_add_cg_props_file(criu_opts *opts, char *path)
 
 int criu_local_add_cg_dump_controller(criu_opts *opts, char *name)
 {
-	char **new;
+	char **new, *ctrl_name;
 	size_t nr;
 
+	ctrl_name = strdup(name);
+	if (!ctrl_name)
+		return -ENOMEM;
+
 	nr = opts->rpc->n_cgroup_dump_controller + 1;
 	new = realloc(opts->rpc->cgroup_dump_controller, nr * sizeof(char *));
-	if (!new)
+	if (!new) {
+		free(ctrl_name);
 		return -ENOMEM;
+	}
 
-	new[opts->rpc->n_cgroup_dump_controller] = strdup(name);
-	if (!new[opts->rpc->n_cgroup_dump_controller])
-		return -ENOMEM;
+	new[opts->rpc->n_cgroup_dump_controller] = ctrl_name;
 
 	opts->rpc->n_cgroup_dump_controller = nr;
 	opts->rpc->cgroup_dump_controller = new;
@@ -1064,9 +1079,16 @@ static int criu_connect(criu_opts *opts, bool d)
 	memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_LOCAL;
 
-	strncpy(addr.sun_path, opts->service_address, sizeof(addr.sun_path));
+	addr_len = strlen(opts->service_address);
+	if (addr_len >= sizeof(addr.sun_path)) {
+		fprintf(stderr, "The service address %s is too long",
+					opts->service_address);
+		close(fd);
+		return -1;
+	}
+	memcpy(addr.sun_path, opts->service_address, addr_len);
 
-	addr_len = strlen(opts->service_address) + sizeof(addr.sun_family);
+	addr_len += sizeof(addr.sun_family);
 
 	ret = connect(fd, (struct sockaddr *) &addr, addr_len);
 	if (ret < 0) {
