@@ -9,7 +9,7 @@ import signal
 import sys
 import struct
 
-import rpc_pb2 as rpc
+import pycriu.rpc_pb2 as rpc
 
 class _criu_comm:
 	"""
@@ -56,7 +56,7 @@ class _criu_comm_sk(_criu_comm):
 
 class _criu_comm_fd(_criu_comm):
 	"""
-	Commnunication class for file descriptor.
+	Communication class for file descriptor.
 	"""
 	def __init__(self, fd):
 		self.comm_type = self.COMM_FD
@@ -85,6 +85,8 @@ class _criu_comm_bin(_criu_comm):
 		css = socket.socketpair(socket.AF_UNIX, socket.SOCK_SEQPACKET)
 		flags = fcntl.fcntl(css[1], fcntl.F_GETFD)
 		fcntl.fcntl(css[1], fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
+		flags = fcntl.fcntl(css[0], fcntl.F_GETFD)
+		fcntl.fcntl(css[0], fcntl.F_SETFD, flags & ~fcntl.FD_CLOEXEC)
 
 		self.daemon = daemon
 
@@ -164,7 +166,7 @@ class CRIUExceptionExternal(CRIUException):
 		s = "%s failed: " % (rpc.criu_req_type.Name(self.typ), )
 
 		if self.typ != self.resp_typ:
-			s += "Unxecpected response type %d: " % (self.resp_typ, )
+			s += "Unexpected response type %d: " % (self.resp_typ, )
 
 		s += "Error(%d): " % (self.errno, )
 
